"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from typing import Any

from .._typecode import Typecode
from ._base import AbstractType


class Binary(AbstractType):
    """
    |result_matrix_desc|

    :py:attr:`.strict_level`
        |strict_level|
    """

    @property
    def typecode(self) -> Typecode:
        return Typecode.STRING

    def __init__(self, value: Any, strict_level: int = 1, **kwargs) -> None:
        super().__init__(value, strict_level, **kwargs)

    def _create_type_checker(self):
        from ..checker import BytesTypeChecker

        return BytesTypeChecker(self._data, self._strict_level)

    def _create_type_converter(self):
        from ..converter import BytesConverter

        return BytesConverter(self._data, self._params)



import os
import sys

import sphinx_rtd_theme

from typepy import __author__, __copyright__, __name__, __version__


sys.path.insert(0, os.path.abspath('../typepy'))


# -- General configuration ------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.todo',
    'sphinx.ext.viewcode',
    'sphinx.ext.intersphinx',
    'sphinx.ext.napoleon',
]

intersphinx_mapping = {'python': ('https://docs.python.org/', None)}


# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = __name__
copyright = __copyright__
author = __author__

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = __version__
# The full version, including alpha/beta/rc tags.
release = version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'pytypeutildoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'typepy.tex', 'typepy Documentation',
     'Tsuyoshi Hombashi', 'manual'),
]


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'typepy', 'typepy Documentation',
     [author], 1)
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'typepy', 'typepy Documentation',
     author, 'typepy', 'One line description of project.',
     'Miscellaneous'),
]


# -- rst_prolog -------------------------------------------
rp_common = """
.. |TM| replace:: :superscript:`TM`
"""

rp_builtin = """
.. |False| replace:: :py:obj:`False`
.. |True| replace:: :py:obj:`True`
.. |None| replace:: :py:obj:`None`
.. |inf| replace:: :py:obj:`inf`
.. |nan| replace:: :py:obj:`nan`

.. |bool| replace:: :py:class:`bool`
.. |dict| replace:: :py:class:`dict`
.. |int| replace:: :py:class:`int`
.. |list| replace:: :py:class:`list`
.. |float| replace:: :py:class:`float`
.. |str| replace:: :py:class:`str`
.. |tuple| replace:: :py:obj:`tuple`
"""

rp_class = """
.. |TypeConversionError| replace:: :py:class:`typepy.TypeConversionError`
.. |DateTime| replace:: :py:class:`typepy.DateTime`
.. |Dictionary| replace:: :py:class:`typepy.Dictionary`
.. |Infinity| replace:: :py:class:`typepy.Infinity`
.. |Integer| replace:: :py:class:`typepy.Integer`
.. |Nan| replace:: :py:class:`typepy.Nan`
.. |NoneType| replace:: :py:class:`typepy.NoneType`
.. |NullString| replace:: :py:class:`typepy.NullString`
.. |RealNumber| replace:: :py:class:`typepy.RealNumber`
.. |String| replace:: :py:class:`typepy.String`
"""

rp_docstring = """
.. |result_matrix_desc| replace::
    For each member methods, the result matrix for each ``strict_level`` is as follows.
    Column headers (except ``Method`` column) indicate input data to ``value`` argument of
    a method in the ``Method`` column.
    For each cell shows the output of the method.

.. |strict_level| replace::
    Represents how much strict to detect the value type. Higher ``strict_level`` means that stricter type check.
"""

rst_prolog = (
    rp_common +
    rp_builtin +
    rp_class +
    rp_docstring
)



#!/usr/bin/env python3

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import os
import sys

import typepy
from typepy import Infinity, Integer, Nan, NullString, RealNumber, StrictLevel, String


nan = float("nan")
inf = float("inf")


class Params:
    def __init__(self, stream):
        self.__stream = stream
        self.strict_level = None
        self.typeclass = None

    def __del__(self):
        if self.__stream:
            self.__stream.close()
            self.__stream = None

    def write_test_params(self, method, value):
        if isinstance(value, str):
            write_value = f'"{value:s}"'
        else:
            write_value = value

        self.__stream.write(
            '["{:s}", {:d}, {}, {}],\n'.format(
                method, self.strict_level, write_value, self.__exeute_method(method, value)
            )
        )

    def __exeute_method(self, method, value):
        try:
            result = getattr(self.typeclass(value, self.strict_level), method)()
            if method == "validate":
                result = "-"
        except (TypeError, typepy.TypeConversionError):
            return '"E"'

        # for string tests
        if NullString(result, StrictLevel.MAX).is_type():
            return '""'

        strict_level = StrictLevel.MAX

        typeobj = Integer(result, strict_level)
        if typeobj.is_type():
            return typeobj.convert()

        typeobj = RealNumber(result, strict_level)
        if typeobj.is_type():
            return typeobj.convert()

        if String(result, strict_level).is_type():
            return f'"{result}"'

        if Infinity(result, strict_level).is_type():
            return '"inf"'

        if Nan(result, strict_level).is_type():
            return '"nan"'

        return result


class TestParamWriter:
    METHOD_LIST = (
        # "is_type",
        # "validate",
        "convert",
        "try_convert",
        "force_convert",
    )

    class Bool:
        VALUE_LIST = [True, "true", 1, 1.1, None]

    class String:
        VALUE_LIST = ["abc", "", 1, "-1", None, True, inf, nan]

    class Number:
        VALUE_LIST = [
            1,
            1.0,
            1.1,
            "0",
            "1.0",
            "1.1",
            sys.maxsize,
            "1,000,000,000,000",
            True,
            None,
            inf,
            nan,
            "test",
            "",
        ]

    def __init__(self, file_path):
        os.makedirs(os.path.dirname(file_path), exist_ok=True)
        self.__writer = Params(open(file_path, "w"))

    def write_bool_tests(self):
        self.__writer.typeclass = typepy.Bool

        for method in self.METHOD_LIST:
            for strict_level in (0, 1, 2):
                self.__writer.strict_level = strict_level
                for value in self.Bool.VALUE_LIST:
                    self.__writer.write_test_params(method, value)

    def write_string_tests(self):
        self.__writer.typeclass = typepy.String

        for method in self.METHOD_LIST:
            for strict_level in (0, 1, 2):
                self.__writer.strict_level = strict_level
                for value in self.String.VALUE_LIST:
                    self.__writer.write_test_params(method, value)

    def write_integer_tests(self):
        self.__writer.typeclass = typepy.Integer

        for method in self.METHOD_LIST:
            for strict_level in (0, 1, 2):
                self.__writer.strict_level = strict_level
                for value in self.Number.VALUE_LIST:
                    self.__writer.write_test_params(method, value)

    def write_realnumber_tests(self):
        self.__writer.typeclass = typepy.RealNumber

        for method in self.METHOD_LIST:
            for strict_level in (0, 1, 2):
                self.__writer.strict_level = strict_level
                for value in self.Number.VALUE_LIST:
                    self.__writer.write_test_params(method, value)


if __name__ == "__main__":
    writer = TestParamWriter("test_cases/integer.txt")
    writer.write_integer_tests()

    writer = TestParamWriter("test_cases/bool.txt")
    writer.write_bool_tests()

    writer = TestParamWriter("test_cases/realnumber.txt")
    writer.write_realnumber_tests()

    writer = TestParamWriter("test_cases/string.txt")
    writer.write_string_tests()



#!/usr/bin/env python3

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import sys

from readmemaker import ReadmeMaker


PROJECT_NAME = "typepy"
OUTPUT_DIR = ".."


def write_examples(maker):
    maker.set_indent_level(0)
    maker.write_chapter("Usage")
    maker.write_introduction_file("usage.txt")


def main():
    maker = ReadmeMaker(
        PROJECT_NAME,
        OUTPUT_DIR,
        is_make_toc=True,
        project_url=f"https://github.com/thombashi/{PROJECT_NAME}",
    )

    maker.write_chapter("Summary")
    maker.write_introduction_file("summary.txt")
    maker.write_introduction_file("badges.txt")

    maker.write_chapter("Features")
    maker.write_introduction_file("features.txt")

    maker.write_introduction_file("installation.rst")

    write_examples(maker)

    maker.set_indent_level(0)
    maker.write_chapter("Documentation")
    maker.write_lines([f"https://{PROJECT_NAME:s}.rtfd.io/"])

    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import argparse
import ipaddress
import os
import sys
from datetime import datetime

import logbook
import pytablewriter as ptw

import typepy
from typepy import (
    Bool,
    DateTime,
    Dictionary,
    Infinity,
    Integer,
    IpAddress,
    List,
    Nan,
    NoneType,
    NullString,
    RealNumber,
    String,
)


logbook.StderrHandler(
    level=logbook.DEBUG, format_string="[{record.level_name}] {record.channel}: {record.message}"
).push_application()

METHOD_HEADER = "Method"


class ExampleWriter:
    _METHOD_LIST = ("is_type", "validate", "convert", "try_convert", "force_convert")


class ResultMatrixWriter(ExampleWriter):
    def __init__(self):
        self.typeclass = None
        self.strict_level = None
        self.headers = None
        self.value_list = None

        self.__table_writer = ptw.RstSimpleTableWriter()
        self.__table_writer._dp_extractor.type_value_map = {
            NullString(None).typecode: '``""``',
            NoneType(None).typecode: "``None``",
            Infinity(None).typecode: '``Decimal("inf")``',
            Nan(None).typecode: '``Decimal("nan")``',
        }
        self.__table_writer.value_map = {
            True: "``True``",
            False: "``False``",
            '``"127.0.0.1"``': '``ip_address("127.0.0.1")``',
        }

    def set_stream(self, output_stream):
        self.__table_writer.stream = output_stream

    def write_type_matrix(self):
        tbl_template = ":py:class:`typepy.{0:s}`: :py:attr:`~typepy.{0:s}.strict_level` = {1:d}"
        self.__table_writer.table_name = tbl_template.format(
            self.typeclass.__name__, self.strict_level
        )
        self.__table_writer.headers = self.headers
        self.__table_writer.value_matrix = self.__get_result_matrix()
        if self.typeclass.__name__ in ["Dictionary", "List"]:
            self.__table_writer.type_hints = [String for _ in self.headers]

        self.__table_writer.write_table()
        self.__table_writer.write_null_line()

    def exeute(self, method, value):
        str_convert_type = (str, ipaddress.IPv4Address, ipaddress.IPv6Address)

        try:
            result = getattr(self.typeclass(value, self.strict_level), method)()

            if method == "validate":
                result = "NOP [#f1]_"
            elif isinstance(result, str_convert_type):
                result = f'``"{result}"``'
        except TypeError:
            result = "E [#f2]_"
        except typepy.TypeConversionError:
            result = "E [#f3]_"

        return result

    def __get_result_matrix(self):
        result_matrix = []
        for method in self._METHOD_LIST:
            result_matrix.append(
                [f":py:meth:`~.type.{self.typeclass.__name__:s}.{method:s}`"]
                + [self.exeute(method, value) for value in self.value_list]
            )

        return result_matrix


class ResultMatrixManager:
    class ExampleBool:
        HEADER = [METHOD_HEADER] + ["``True``", '``"true"``', 1]
        VALUE = [True, "true", 1]

    class ExampleString:
        HEADER = [METHOD_HEADER] + ['``"abc"``', '``""``', '``"  "``', "``None``", "``1``"]
        VALUE = ["abc", "", "  ", None, 1]

    class ExampleNumber:
        HEADER = [METHOD_HEADER] + [
            "``1``",
            "``1.0``",
            "``1.1``",
            '``"1"``',
            '``"1.0"``',
            '``"1.1"``',
            "``True``",
        ]
        VALUE = [1, 1.0, 1.1, "1", "1.0", "1.1", True]

    class ExampleInfinity:
        HEADER = [METHOD_HEADER] + ['``float("inf")``', '``"Infinity"``', "``0.1``"]
        VALUE = [float("inf"), "Infinity", 0.1]

    class ExampleNan:
        HEADER = [METHOD_HEADER] + ['``float("nan")``', '``"NaN"``', "``0.1``"]
        VALUE = [float("nan"), "NaN", 0.1]

    class ExampleDateTime:
        HEADER = [METHOD_HEADER] + [
            "``datetime(2017, 1, 23, 4, 56)``",
            '``"2017-01-22T04:56:00+0900"``',
            "``1485685623``",
            '``"1485685623"``',
        ]
        VALUE = [datetime(2017, 1, 23, 4, 56), "2017-01-22T04:56:00+0900", 1485685623, "1485685623"]

    class ExampleIpAddress:
        HEADER = [METHOD_HEADER] + [
            "``ip_address('127.0.0.1')``",
            "``'127.0.0.1'``",
            "``'::1'``",
            "``'192.168.0.256'``",
            "``None``",
        ]
        VALUE = [ipaddress.ip_address("127.0.0.1"), "127.0.0.1", "::1", "192.168.0.256", None]

    class ExampleList:
        HEADER = [METHOD_HEADER] + [
            "``[]``",
            '``["a", "b"]``',
            '``("a", "b")``',
            '``{"a": 1}``',
            '``"abc"``',
        ]
        VALUE = [[], ["a", "b"], ("a", "b"), {"a": 1}, "abc"]

    class ExampleDictionary:
        HEADER = [METHOD_HEADER] + ["``{}``", '``{"a": 1}``', '``(("a", 1), )``']
        VALUE = [{}, {"a": 1}, (("a", 1),)]

    def __init__(self):
        self.strict_level_list = None

        self.__ewriter = ResultMatrixWriter()

    def set_stream(self, output_stream):
        self.__ewriter.set_stream(output_stream)

    def write_none(self):
        self.__ewriter.typeclass = NoneType
        self.__ewriter.headers = self.ExampleString.HEADER
        self.__ewriter.value_list = self.ExampleString.VALUE
        self.__write()

    def write_ipaddress(self):
        self.__ewriter.typeclass = IpAddress
        self.__ewriter.headers = self.ExampleIpAddress.HEADER
        self.__ewriter.value_list = self.ExampleIpAddress.VALUE
        self.__write()

    def write_bool(self):
        self.__ewriter.typeclass = Bool
        self.__ewriter.headers = self.ExampleBool.HEADER
        self.__ewriter.value_list = self.ExampleBool.VALUE
        self.__write()

    def write_string(self):
        self.__ewriter.typeclass = String
        self.__ewriter.headers = self.ExampleString.HEADER
        self.__ewriter.value_list = self.ExampleString.VALUE
        self.__write()

    def write_null_string(self):
        self.__ewriter.typeclass = NullString
        self.__ewriter.headers = self.ExampleString.HEADER
        self.__ewriter.value_list = self.ExampleString.VALUE
        self.__write()

    def write_integer(self):
        self.__ewriter.typeclass = Integer
        self.__ewriter.headers = self.ExampleNumber.HEADER
        self.__ewriter.value_list = self.ExampleNumber.VALUE
        self.__write()

    def write_realnumber(self):
        self.__ewriter.typeclass = RealNumber
        self.__ewriter.headers = self.ExampleNumber.HEADER
        self.__ewriter.value_list = self.ExampleNumber.VALUE
        self.__write()

    def write_infinity(self):
        self.__ewriter.typeclass = Infinity
        self.__ewriter.headers = self.ExampleInfinity.HEADER
        self.__ewriter.value_list = self.ExampleInfinity.VALUE
        self.__write()

    def write_nan(self):
        self.__ewriter.typeclass = Nan
        self.__ewriter.headers = self.ExampleNan.HEADER
        self.__ewriter.value_list = self.ExampleNan.VALUE
        self.__write()

    def write_datetime(self):
        self.__ewriter.typeclass = DateTime
        self.__ewriter.headers = self.ExampleDateTime.HEADER
        self.__ewriter.value_list = self.ExampleDateTime.VALUE
        self.__write()

    def write_list(self):
        self.__ewriter.typeclass = List
        self.__ewriter.headers = self.ExampleList.HEADER
        self.__ewriter.value_list = self.ExampleList.VALUE
        self.__write()

    def write_dictionary(self):
        self.__ewriter.typeclass = Dictionary
        self.__ewriter.headers = self.ExampleDictionary.HEADER
        self.__ewriter.value_list = self.ExampleDictionary.VALUE
        self.__write()

    def __write(self):
        for strict_level in self.strict_level_list:
            self.__ewriter.strict_level = strict_level
            self.__ewriter.write_type_matrix()


class PathMaker:
    def __init__(self, output_dir, encoding="utf8"):
        self.__output_dir = output_dir
        self.__encoding = encoding

        if not os.path.isdir(output_dir):
            os.makedirs(output_dir)

    def open_write(self, filename):
        file_path = os.path.join(self.__output_dir, filename)
        print(file_path)

        return open(file_path, "w", encoding=self.__encoding)


def parse_option():
    parser = argparse.ArgumentParser()

    parser.add_argument("--type", choices=["reference", "example"])

    parser.add_argument(
        "-o", "--output-dir", default=".", required=True, help="default=%(default)s"
    )

    return parser.parse_args()


def make_filename(name, prefix="matrix_", suffix="_type", extension=".txt"):
    return f"{prefix:s}{name:s}{suffix:s}{extension:s}"


def write_result_matrix(output_dir_path):
    manager = ResultMatrixManager()
    opener = PathMaker(output_dir_path)

    manager.strict_level_list = (0,)
    with opener.open_write(make_filename("none")) as f:
        manager.set_stream(f)
        manager.write_none()

    manager.strict_level_list = (0, 1)
    with opener.open_write(make_filename("ipaddress")) as f:
        manager.set_stream(f)
        manager.write_ipaddress()

    with opener.open_write(make_filename("infinity")) as f:
        manager.set_stream(f)
        manager.write_infinity()

    with opener.open_write(make_filename("nan")) as f:
        manager.set_stream(f)
        manager.write_nan()

    with opener.open_write(make_filename("list")) as f:
        manager.set_stream(f)
        manager.write_list()

    with opener.open_write(make_filename("dictionary")) as f:
        manager.set_stream(f)
        manager.write_dictionary()

    with opener.open_write(make_filename("nullstring")) as f:
        manager.set_stream(f)
        manager.write_null_string()

    manager.strict_level_list = (0, 1, 2)
    with opener.open_write(make_filename("bool")) as f:
        manager.set_stream(f)
        manager.write_bool()

    with opener.open_write(make_filename("datetime")) as f:
        manager.set_stream(f)
        manager.write_datetime()

    with opener.open_write(make_filename("integer")) as f:
        manager.set_stream(f)
        manager.write_integer()

    with opener.open_write(make_filename("realnumber")) as f:
        manager.set_stream(f)
        manager.write_realnumber()

    with opener.open_write(make_filename("string")) as f:
        manager.set_stream(f)
        manager.write_string()


def write_example(output_dir_path):
    opener = PathMaker(output_dir_path)  # noqa


def main():
    options = parse_option()

    # ptw.set_log_level(logbook.DEBUG)

    write_result_matrix(options.output_dir)
    write_example(options.output_dir)

    return 0


if __name__ == "__main__":
    sys.exit(main())
